import {
  timeParse,
  utcWeek,
  utcWeeks,
  group,
  stack,
  extent,
  stackOffsetWiggle,
  stackOrderInsideOut,
} from 'd3';
import { blur } from 'array-blur';

const parseDate = timeParse('%Y-%m-%d');

// These projects are not really "Stamen Projects" per se.
// Stamen just so happened to fork them.
const blacklist = new Set([
  'mapbox-gl-js',
  'openmaptiles',
  'xyz-showcase',
  'WhirlyGlobe',
  'react-fullpage',
  'react-dropdown',
  'react-autosuggest',
  'react-slick',
  'natural-earth-vector',
  'react-typeahead',
  'openstreetmap-carto',
  'intro.js',
  'leaflet-rails',
  'osm',
  'GetOutside',
  'Cinder-MsKinect',
  'tm2'
]);

const layer = (d) => d.repo;

export const transformData = (data) => {
  data.forEach((d) => {
    d.date = utcWeek.floor(parseDate(d.date.split(' ')[0]));
  });

  // Aggregate by month and repository.
  const groupedData = group(data, (d) => d.date, layer);

  const layerGroupedData = group(data, layer);

  const layers = Array.from(layerGroupedData.keys()).filter(
    (layer) => !blacklist.has(layer)
  );

  const [start, stop] = extent(data, (d) => d.date);
  const allWeeks = utcWeeks(start, stop);

  const dataBylayer = new Map();

  for (let layer of layers) {
    const layerData = allWeeks.map((date) => {
      const value = groupedData.get(date);
      const commits = value ? value.get(layer) : null;
      const commitCount = commits ? commits.length : 0;
      return commitCount;
    });

    // Apply smoothing
    const smoothedLayerData = blur().radius(13)(layerData);

    dataBylayer.set(layer, smoothedLayerData);
  }

  const transformedData = [];
  allWeeks.forEach((date, i) => {
    const row = { date };
    for (let layer of layers) {
      row[layer] = dataBylayer.get(layer)[i];
    }
    transformedData.push(row);
  });

  const stackedData = stack()
    .offset(stackOffsetWiggle)
    .order(stackOrderInsideOut)
    .keys(layers)(transformedData);

  return { data, stackedData };
};
